import java.time.*;
import java.util.*;
import org.junit.*;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

class MockPowerSource implements PowerSource {
    List<Integer> values = new LinkedList<>();
    List<Instant> times = new LinkedList<>();

    public MockPowerSource() {}

    public void set(int amount) {
        this.values.add(amount);
        this.times.add(Instant.now());
    }
}

public class SampleTest {
    @Test
    public void testHeater() {
        Mode[] modes = new Mode[] {Mode.OFF, Mode.ON, Mode.DIM};
        MockPowerSource powerSource = new MockPowerSource();
        Duration closeThreshold = Duration.ofMillis(50);
        Duration timeout = Duration.ofMillis(500);

        Heater heater = new Heater(
            modes,
            timeout,
            closeThreshold,
            powerSource
        );

        assertEquals(Mode.OFF, heater.getCurrentMode());
        assertEquals(List.of(0), powerSource.values);

        heater.click();

        assertEquals(Mode.ON, heater.getCurrentMode());
        assertEquals(List.of(0, 100), powerSource.values);
    }

    private static Simulator.Event e(long ms, int count, Mode oldMode, Mode newMode) {
        return new Simulator.Event(
                   Instant.now().plusMillis(ms),
                   count,
                   oldMode,
                   newMode
               );
    }

    @Test
    public void testSimulator() {
        Mode[] modes = new Mode[] {Mode.OFF, Mode.ON, Mode.DIM};
        PowerSource powerSource = new MockPowerSource();
        Duration closeThreshold = Duration.ofMillis(50);
        Duration timeout = Duration.ofMillis(500);

        Heater heater = new Heater(
            modes,
            timeout,
            closeThreshold,
            powerSource
        );

        Simulator simulator = new Simulator(
            heater,
            Simulator.Strategy.SEQUENTIAL,
            List.of(
                e(100, 2, Mode.OFF, Mode.ON),
                e(200, 2, Mode.ON, Mode.DIM),
                e(400, 2, Mode.DIM, Mode.OFF),
                e(1000, 4, Mode.OFF, Mode.ON),
                e(1600, 4, Mode.ON, Mode.OFF)
            )
        );

        assertTrue(simulator.run());
    }
}
